/*************************************************** 
  This is a example sketch demonstrating bitmap drawing
  capabilities of the SSD1351 library for the 1.5" 
  and 1.27" 16-bit Color OLEDs with SSD1351 driver chip

  Pick one up today in the adafruit shop!
  ------> http://www.adafruit.com/products/1431
  ------> http://www.adafruit.com/products/1673
 
  If you're using a 1.27" OLED, change SSD1351HEIGHT in Adafruit_SSD1351.h
 	to 96 instead of 128

  These displays use SPI to communicate, 4 or 5 pins are required to  
  interface
  Adafruit invests time and resources providing this open source code, 
  please support Adafruit and open-source hardware by purchasing 
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.  
  BSD license, all text above must be included in any redistribution

  The Adafruit GFX Graphics core library is also required
  https://github.com/adafruit/Adafruit-GFX-Library
  Be sure to install it!
 ****************************************************/

#include <Adafruit_GFX.h>
#include <Adafruit_SSD1351.h>
//#include <SD.h>
#include <SPI.h>


// If we are using the hardware SPI interface, these are the pins (for future ref)
//#define my_sclk 13
//#define my_mosi 11
#define my_dc   8
#define my_cs   9
#define my_rst  7

// Color definitions
#define	BLACK           0x0000
#define	BLUE            0x001F
#define	RED             0xF800
#define	GREEN           0x07E0
#define CYAN            0x07FF
#define MAGENTA         0xF81F
#define YELLOW          0xFFE0  
#define WHITE           0xFFFF
 
// to draw images from the SD card, we will share the hardware SPI interface
Adafruit_SSD1351 tft = Adafruit_SSD1351(my_cs, my_dc, my_rst);

#define ATN 38        // Defined because the IDE version I'm using doesn't define this yet.
#define SD_CS ATN     // Default CS pin for the Zero's SPI bus is ATN, which is pin 38.

// the file itself
//File bmpFile;

// information we extract about the bitmap file
int bmpWidth, bmpHeight;
uint8_t bmpDepth, bmpImageoffset;

unsigned long startTime, endTime; // For calculating FPS.
unsigned int frames;

void setup(void) {
  //SerialUSB.begin(9600);
   
  //pinMode(my_cs, OUTPUT);
  //digitalWrite(my_cs, HIGH);

  pinMode(13, INPUT_PULLUP); // Illuminate status LED.
     
  // initialize the OLED
  tft.begin();

/*
  tft.writeCommand(SSD1351_CMD_SETVSL);
  tft.writeData(B10100010);
  tft.writeData(B10110101); 
  tft.writeData(B01010101);
*/
  
  //delay(5000);
  //SerialUSB.println("init");
  
  //tft.fillScreen(BLUE);
  
  //delay(500);
  //SerialUSB.print("Initializing SD card...");

  //if (!SD.begin(SD_CS)) {
  //  SerialUSB.println("failed!");
  //  return;
  //}
  //SerialUSB.println("SD OK!");

  //SPI.setClockDivider(4); // Set SPI clock to 12MHz (max on SAMD21)
  
}

void loop() {

    cls(BLACK);

/*
    // Bitmaps:
    
      startTime = millis();
      frames = 0;
      
      for (int n = 0; n < 1; n++) { // .46 FPS before SPI_FULL_SPEED was added to SDClass::begin in SD.cpp in SD library.  .56 fps after.  1.46 FPS after adding block transfer to SD lib.

        bmpDraw("1.bmp", 0, 0);
        bmpDraw("2.bmp", 0, 0);
        bmpDraw("3.bmp", 0, 0);
        
        frames = frames + 3;
        
      }
      
      endTime = millis();
      showFPS("24-bit BMPs:"); 

  cls(BLACK);
  
  // Raw bitmaps:

      startTime = millis();
      frames = 0;
      
      for (int n = 0; n < 5; n++) { // 7.15 FPS after optimizations to SD lib.

        rawFullSPI("1.bin");
        rawFullSPI("2.bin");
        rawFullSPI("3.bin");

        frames = frames + 3;
        
      }
      
      endTime = millis();
      showFPS("Raw 16-bit bitmaps:"); 
*/

   // Full screen rectangles:

      startTime = millis();
      frames = 0;
      
      for (int n = 0; n < 5; n++) { 
    
        cls(RED); 
        cls(GREEN);
        cls(BLUE);
        
        frames = frames + 3;
        
      }
      
      endTime = millis();
      showFPS("Full screen rects:");
  
  
    // Random rectangles:
    // No benchmark, because they're random!

/*
      cls(BLACK);
      tft.setCursor(0, 5);
      tft.setTextColor(RED);  
      tft.setTextSize(1);
      tft.println(F("Drawing 500 rects..."));
      delay(500);
*/
      
    
      cls(BLACK);
      
      // Draw rectangles of random color and size:
      for (int n = 0; n<200; n++) {
        byte c = random(0, 3);
        uint16_t color = BLACK;
        if (c == 0) { color = RED; }
        if (c == 1) { color = GREEN; }
        if (c == 2) { color = BLUE; } 
        fastRect(random(0, 128), random(0, 128), random(0, 128), random(0, 128), color);
      }    

  /*
  cls(BLACK);
  rawFullSPI("1.bin");
  delay(500);

  tft.invert(true);
  delay(100);
  tft.invert(false);
  delay(100);
  tft.invert(true);
  delay(100);
  tft.invert(false);
  delay(100);
  tft.invert(true);
  delay(100);
  tft.invert(false);
  delay(100);
  */
            
  cls(BLACK);
  lcdTestPattern();
  delay(500);

  cls(BLACK);
  testdrawtext("Lorem ipsum dolor sit amet, consectetur adipiscing elit. Curabitur adipiscing ante sed nibh tincidunt feugiat. Maecenas enim massa, fringilla sed malesuada et, malesuada sit amet turpis. Sed porttitor neque ut ante pretium vitae malesuada nunc bibendum. Nullam aliquet ultrices massa eu hendrerit. Ut sed nisi lorem. In vestibulum purus a tortor imperdiet posuere.", WHITE);
  delay(500);

  // tft print function!
  tftPrintTest1();
  //delay(1000);

  tftPrintTest2();
  delay(1000);
  
  //a single pixel
  //tft.drawPixel(tft.width()/2, tft.height()/2, GREEN);
  //delay(500);

  // line draw test
  testlines(CYAN, WHITE); 
  delay(500);    
 
  // optimized lines
  testfastlines(RED, BLUE);
  delay(500);    

  testdrawrects(GREEN);
  delay(1000);

  testfillrects(RED, BLUE);
  delay(1000);

  cls(BLACK);
  testfillcircles(8, BLUE);
  testdrawcircles(8, WHITE);
  delay(1000);

   /*
  testroundrects();
  delay(500);
  
  testtriangles();
  delay(500);     
    */
      
}


